@@ -2,11 +2,11 @@
 //! # cargo-auto  
 //!
 //! **cargo-auto - automation tasks written in Rust language for the build process of rust projects**  
-//! ***[repository](https://github.com/LucianoBestia/cargo-auto); version: 2021.816.1211  date: 2021-08-16 authors: Luciano Bestia***  
+//! ***[repository](https://github.com/LucianoBestia/cargo-auto); version: 2021.819.906  date: 2021-08-19 authors: Luciano Bestia***  
 //!
-//! [![Lines in Rust code](https://img.shields.io/badge/Lines_in_Rust-420-green.svg)](https://github.com/LucianoBestia/cargo-auto/)
-//! [![Lines in Doc comments](https://img.shields.io/badge/Lines_in_Doc_comments-274-blue.svg)](https://github.com/LucianoBestia/cargo-auto/)
-//! [![Lines in Comments](https://img.shields.io/badge/Lines_in_comments-47-purple.svg)](https://github.com/LucianoBestia/cargo-auto/)
+//! [![Lines in Rust code](https://img.shields.io/badge/Lines_in_Rust-584-green.svg)](https://github.com/LucianoBestia/cargo-auto/)
+//! [![Lines in Doc comments](https://img.shields.io/badge/Lines_in_Doc_comments-280-blue.svg)](https://github.com/LucianoBestia/cargo-auto/)
+//! [![Lines in Comments](https://img.shields.io/badge/Lines_in_comments-56-purple.svg)](https://github.com/LucianoBestia/cargo-auto/)
 //! [![Lines in examples](https://img.shields.io/badge/Lines_in_examples-0-yellow.svg)](https://github.com/LucianoBestia/cargo-auto/)
 //! [![Lines in tests](https://img.shields.io/badge/Lines_in_tests-0-orange.svg)](https://github.com/LucianoBestia/cargo-auto/)
 //!
@@ -143,6 +143,10 @@
 //! The code-flow of the source code of `cargo-auto` is simple, fully commented and straightforward to audit.  
 //! The source code is on [GitHub](https://github.com/LucianoBestia/cargo-auto) with MIT open-source licensing.  
 //!
+//! ## bash auto-completion
+//!
+//! With the help of the crate [dev_bestia_cargo_completion](https://crates.io/crates/dev_bestia_cargo_completion) the commands `cargo` and `cargo auto` get bash auto-completion. Try it and maybe contribute to add the commands you use most.  
+//!
 //! ## templates
 //!
 //! Inside the cargo-auto project there are 2 directories with rust sub-projects as templates. I can open a new editor for these directories and build this crates independently. So it is easy to debug and develop.  
@@ -317,7 +321,9 @@ fn match_first_argument(task: &str, mut args: std::env::Args) {
             // early exit
             std::process::exit(0);
         }
-        auto_new(&mut args);
+        auto_new(&mut args, task);
+    } else if task == "completion" {
+        completion();
     } else {
         if !already_exists_automation_tasks_rs() {
             println!("{}Error: Directory automation_tasks_rs does not exist.{}", *RED, *RESET);
@@ -337,6 +343,40 @@ fn match_first_argument(task: &str, mut args: std::env::Args) {
     }
 }
 
+/// sub-command for bash auto-completion of `cargo auto` using the crate `dev_bestia_cargo_completion`
+fn completion() {
+    /// println one, more or all sub_commands
+    fn completion_return_one_or_more_sub_commands(sub_commands: Vec<&str>, word_being_completed: &str) {
+        let mut sub_found = false;
+        for sub_command in sub_commands.iter() {
+            if sub_command.starts_with(word_being_completed) {
+                println!("{}", sub_command);
+                sub_found = true;
+            }
+        }
+        if sub_found == false {
+            // print all sub-commands
+            for sub_command in sub_commands.iter() {
+                println!("{}", sub_command);
+            }
+        }
+    }
+
+    let args: Vec<String> = std::env::args().collect();
+    let last_word = args[2].as_str();
+    let mut word_being_completed = " ";
+    if args.len() > 3 {
+        word_being_completed = args[3].as_str();
+    }
+    if last_word == "cargo-auto" || last_word == "auto" {
+        let sub_commands = vec!["new"];
+        completion_return_one_or_more_sub_commands(sub_commands, word_being_completed);
+    } else if last_word == "new" {
+        let sub_commands = vec!["with_lib"];
+        completion_return_one_or_more_sub_commands(sub_commands, word_being_completed);
+    }
+}
+
 /// build if the date of Cargo.toml or main.rs is newer then of automation_tasks_rs/target/automation_tasks_rs
 fn build_automation_tasks_rs_if_needed() {
     if !PATH_TARGET_DEBUG_AUTOMATION_TASKS_RS.exists() {
@@ -379,13 +419,21 @@ fn already_exists_automation_tasks_rs() -> bool {
 /// with the argument `with_lib` copies template_with_lib  
 /// in development use: `cargo run -- new with_lib`  
 /// in runtime use: `cargo auto new with_lib`  
-fn auto_new(args: &mut std::env::Args) {
-    let arg_2 = args.next();
-    match arg_2 {
-        None => copy_template("basic"),
-        Some(template_name) => copy_template(&template_name),
+fn auto_new(args: &mut std::env::Args, task: &str) {
+    let template_name: String;
+    if task == "new" {
+        let arg_2 = args.next();
+        match arg_2 {
+            None => template_name = "basic".to_string(),
+            Some(template_name_a) => template_name = template_name_a,
+        }
+    } else {
+        template_name = task.trim_start_matches("new_").to_string();
     }
+    copy_template(&template_name);
+
     build_automation_tasks_rs_if_needed();
+
     println!("");
     println!("`crate auto new` generated the directory `automation_tasks_rs` in your main rust project.");
     println!("You can open this new helper rust project in a new rust editor.");
